import csv
import datetime

from sqlalchemy import Engine, select
from sqlalchemy.orm import Session

from nupes.cache import get_file
from nupes.models import Genre, Personnalite
from nupes.models.europeennes2024 import BlocEuropeennes2024, NuanceEuropeennes2024, ListeEuropeennes2024, \
    CandidatEuropeennes2024


def creer_blocs(engine: Engine, verbose: bool = False) -> None:
    blocs = [
        {"id": 1, "nom": "Gauche", "couleur": "#BB1840"},
        {"id": 2, "nom": "Droite libérale", "couleur": "#FFEB00"},
        {"id": 3, "nom": "Droite conservatrice", "couleur": "#0066CC"},
        {"id": 4, "nom": "Extrême droite", "couleur": "#404040"},
        {"id": 5, "nom": "Autres", "couleur": "#DCBFA3"}
    ]

    with Session(engine) as session:
        for bloc_dict in blocs:
            if bloc := session.execute(select(BlocEuropeennes2024).filter_by(id=bloc_dict["id"])).scalar_one_or_none():
                bloc.nom = bloc_dict["nom"]
                bloc.couleur = bloc_dict["couleur"]
            else:
                session.add(BlocEuropeennes2024(**bloc_dict))

        session.commit()


def creer_nuances(engine: Engine, verbose: bool = False) -> None:
    nuances = [
        {"code": "LEXG", "nom": "Liste d'extrême gauche", "couleur": "#BB0000"},
        {"code": "LCOM", "nom": "Liste du Parti communiste français", "couleur": "#DD0000"},
        {"code": "LFI", "nom": "Liste de La France insoumise", "couleur": "#CC2443"},
        {"code": "LUG", "nom": "Liste d'union de la gauche", "couleur": "#FF8080"},
        {"code": "LVEC", "nom": "Liste Les Écologistes", "couleur": "#00C000"},
        {"code": "LDVG", "nom": "Liste divers gauche", "couleur": "#FFC0C0"},
        {"code": "LECO", "nom": "Liste écologiste", "couleur": "#77FF77"},
        {"code": "LDIV", "nom": "Liste Divers", "couleur": "#DCDCDC"},
        {"code": "LENS", "nom": "Liste Ensemble", "couleur": "#FFEB00"},
        {"code": "LLR", "nom": "Liste des Républicains", "couleur": "#0066CC"},
        {"code": "LDVD", "nom": "Liste divers droite", "couleur": "#26C4EC"},
        {"code": "LRN", "nom": "Liste du Rassemblement national", "couleur": "#0D378A"},
        {"code": "LREC", "nom": "Liste Reconquête !", "couleur": "#404040"},
        {"code": "LEXD", "nom": "Liste d'extrême droite", "couleur": "#404040"},
    ]

    with Session(engine) as session:
        for nuance_dict in nuances:
            if nuance := session.execute(select(NuanceEuropeennes2024)
                                                 .filter_by(code=nuance_dict["code"])).scalar_one_or_none():
                nuance.nom = nuance_dict["nom"]
                nuance.couleur = nuance_dict["couleur"]
            else:
                session.add(NuanceEuropeennes2024(**nuance_dict))

        session.commit()


def importer_listes(engine: Engine, verbose: bool = False) -> None:
    listes = [
        {"numero": 1, "nom": "Pour une humanité souveraine", "nuance_id": "LDIV", "bloc_id": 5},
        {"numero": 2, "nom": "Pour une démocratie réelle : Décidons nous-mêmes !",
         "nuance_id": "LDIV", "bloc_id": 5},
        {"numero": 3, "nom": "La France fière, menée par Marion Maréchal et soutenue par Éric Zemmour",
         "nuance_id": "LREC", "bloc_id": 4},
        {"numero": 4, "nom": "La France insoumise - Union populaire", "nuance_id": "LFI", "bloc_id": 1},
        {"numero": 5, "nom": "La France revient ! Avec Jordan Bardella et Marine Le Pen",
         "nuance_id": "LRN", "bloc_id": 4},
        {"numero": 6, "nom": "Europe Écologie", "nuance_id": "LVEC", "bloc_id": 1},
        {"numero": 7, "nom": "Free Palestine", "nuance_id": "LDIV", "bloc_id": 1},
        {"numero": 8, "nom": "Parti animaliste - Les animaux comptent, votre voix aussi",
         "nuance_id": "LDIV", "bloc_id": 5},
        {"numero": 9, "nom": "Parti révolutionnaire Communistes", "nuance_id": "LEXG", "bloc_id": 1},
        {"numero": 10, "nom": "Parti pirate", "nuance_id": "LDIV", "bloc_id": 1},
        {"numero": 11, "nom": "Besoin d'Europe", "nuance_id": "LENS", "bloc_id": 2},
        {"numero": 12, "nom": "PACE - Parti des citoyens européens, pour l'armée européenne, "
                              "pour l'Europe sociale, pour la planète !", "nuance_id": "LDIV", "bloc_id": 1},
        {"numero": 13, "nom": "Équinoxe : écologie pratique et renouveau démocratique",
         "nuance_id": "LECO", "bloc_id": 5},
        {"numero": 14, "nom": "Écologie positive et territoires", "nuance_id": "LECO", "bloc_id": 5},
        {"numero": 15, "nom": "Liste Asselineau-Frexit, pour le pouvoir d'achat et pour la paix",
         "nuance_id": "LDIV", "bloc_id": 4},
        {"numero": 16, "nom": "Paix et Décroissance", "nuance_id": "LEXG", "bloc_id": 1},
        {"numero": 17, "nom": "Pour une autre Europe", "nuance_id": "LDIV", "bloc_id": 5},
        {"numero": 18, "nom": "La droite pour faire entendre la voix de la France en Europe",
         "nuance_id": "LLR", "bloc_id": 3},
        {"numero": 19, "nom": "Lutte ouvrière le camp des travailleurs", "nuance_id": "LEXG", "bloc_id": 1},
        {"numero": 20, "nom": "Changer l'Europe", "nuance_id": "LDVG", "bloc_id": 1},
        {"numero": 21, "nom": "Nous le peuple", "nuance_id": "LDIV", "bloc_id": 5},
        {"numero": 22, "nom": "Pour un monde sans frontières ni patrons, urgence révolution !",
         "nuance_id": "LEXG", "bloc_id": 1},
        {"numero": 23, "nom": "\"Pour le pain, la paix, la liberté !\" présentée par le Parti des travailleurs",
         "nuance_id": "LEXG", "bloc_id": 1},
        {"numero": 24, "nom": "L'Europe ça suffit !", "nuance_id": "LEXD", "bloc_id": 4},
        {"numero": 25, "nom": "Non ! Prenons-nous en mains", "nuance_id": "LDIV", "bloc_id": 4},
        {"numero": 26, "nom": "Forteresse Europe — Liste d'unité nationaliste", "nuance_id": "LEXD", "bloc_id": 4},
        {"numero": 27, "nom": "Réveiller l'Europe", "nuance_id": "LUG", "bloc_id": 1},
        {"numero": 28, "nom": "Non à l'UE et à l'OTAN, communistes pour la paix et le progrès social",
         "nuance_id": "LEXG", "bloc_id": 1},
        {"numero": 29, "nom": "Alliance rurale", "nuance_id": "LDVD", "bloc_id": 3},
        {"numero": 30, "nom": "France libre", "nuance_id": "LDIV", "bloc_id": 4},
        {"numero": 31, "nom": "Europe Territoires Écologie", "nuance_id": "LDVG", "bloc_id": 1},
        {"numero": 32, "nom": "La ruche citoyenne", "nuance_id": "LDIV", "bloc_id": 5},
        {"numero": 33, "nom": "Gauche unie pour le monde du travail soutenue par Fabien Roussel",
         "nuance_id": "LCOM", "bloc_id": 1},
        {"numero": 34, "nom": "Défendre les enfants", "nuance_id": "LDIV", "bloc_id": 5},
        {"numero": 35, "nom": "Écologie au centre", "nuance_id": "LECO", "bloc_id": 2},
        {"numero": 36, "nom": "Démocratie représentative", "nuance_id": "LDIV", "bloc_id": 1},
        {"numero": 37, "nom": "Espéranto langue commune", "nuance_id": "LDIV", "bloc_id": 5},
        {"numero": 38, "nom": "Liberté démocratique française", "nuance_id": "LDIV", "bloc_id": 4},
    ]

    with Session(engine) as session:
        for liste_dict in listes:
            if liste := session.execute(select(ListeEuropeennes2024)
                                                .filter_by(numero=liste_dict["numero"])).scalar_one_or_none():
                liste.nom = liste_dict["nom"]
                liste.nuance_id = liste_dict["nuance_id"]
                liste.bloc_id = liste_dict["bloc_id"]
            else:
                liste = ListeEuropeennes2024(numero=liste_dict['numero'], nom=liste_dict['nom'],
                                             nom_majuscules=liste_dict['nom'].upper(),
                                             nuance_id=liste_dict['nuance_id'], bloc_id=liste_dict['bloc_id'])
                session.add(liste)

        session.commit()


def importer_candidats(engine: Engine, verbose: bool = False) -> None:
    DATASET_URL = "https://www.data.gouv.fr/fr/datasets/r/483cd4bd-1b0e-4b52-a923-aadadf9c8f62"
    file = get_file(DATASET_URL, "candidats_2024.csv")

    with file.open('r') as f:
        with Session(engine) as session:
            reader = csv.DictReader(f, delimiter=';')
            for line in reader:
                line: dict[str, str]
                numero_panneau = line.get("Numéro de panneau")
                liste = session.execute(select(ListeEuropeennes2024).filter_by(numero=numero_panneau)).scalar_one()
                liste.nom_majuscules = line['Libellé de la liste']

                if candidat := session.execute(select(CandidatEuropeennes2024)
                                               .filter_by(liste_id=liste.id, ordre=line['Ordre'])) \
                        .scalar_one_or_none():
                    candidat.liste_id = liste.id
                    candidat.nom = line['Nom sur le bulletin de vote']
                    candidat.prenom = line['Prénom sur le bulletin de vote']
                    candidat.sexe = Genre(line['Sexe'])
                    candidat.date_naissance = datetime.datetime.strptime(line['Date de naissance'], "%d/%m/%Y").date()
                    candidat.profession = line['Profession']
                    candidat.code_personnalite = Personnalite(line['Code personnalité'])
                    candidat.sortant = line['Sortant'] == "OUI"
                else:
                    candidat = CandidatEuropeennes2024(
                        liste_id=liste.id,
                        ordre=int(line['Ordre']),
                        nom=line['Nom sur le bulletin de vote'],
                        prenom=line['Prénom sur le bulletin de vote'],
                        sexe=Genre(line['Sexe']).name,
                        date_naissance=datetime.datetime.strptime(line['Date de naissance'], "%d/%m/%Y").date(),
                        profession=line['Profession'],
                        code_personnalite=Personnalite(line['Code personnalité']).name,
                        sortant=line['Sortant'] == "OUI",
                    )
                    session.add(candidat)

            session.commit()


def run(engine: Engine, verbose: bool = False) -> None:
    creer_blocs(engine, verbose)
    creer_nuances(engine, verbose)
    importer_listes(engine, verbose)
    importer_candidats(engine, verbose)
